@@ -47,7 +47,7 @@ use Carp qw/croak carp confess cluck/;
 use Scalar::Util qw/looks_like_number blessed reftype/;
 use Unicode::UTF8 qw/decode_utf8 valid_utf8 encode_utf8/;
 use B;
-our $VERSION = '0.30_05';
+our $VERSION = '0.30_06';
 
 sub create_json
 {
@@ -108,11 +108,12 @@ sub isfloat
     return $isfloat;
 }
 
-# This is for compatibility with JSON::Parse.
+# Built in booleans. The nasty PL_sv_(yes|no) stuff comes from
+# JSON::Parse. The JSON::Create::Bool is from our own nice module.
 
 sub isbool
 {
-    my ($ref) = @_;
+    my ($input, $ref) = @_;
     my $poo = B::svref_2object ($ref);
     if (ref $poo eq 'B::SPECIAL') {
 	# Leave the following commented-out code as reference for what
@@ -318,9 +319,69 @@ sub comma
     }
 }
 
-sub create_json_recursively
+sub array
 {
     my ($jc, $input) = @_;
+    $jc->openB ('[');
+    my $i = 0;
+    for my $k (@$input) {
+	if ($i != 0) {
+	    $jc->comma ();
+	}
+	$i++;
+	my $error = create_json_recursively ($jc, $k, \$k);
+	if ($error) {
+	    return $error;
+	}
+    }
+    $jc->closeB (']');
+    return undef;
+}
+
+sub object
+{
+    my ($jc, $input) = @_;
+    $jc->openB ('{');
+    my @keys = keys %$input;
+    if ($jc->{_sort}) {
+	if ($jc->{cmp}) {
+	    @keys = sort {&{$jc->{cmp}} ($a, $b)} @keys;
+	}
+	else {
+	    @keys = sort @keys;
+	}
+    }
+    my $i = 0;
+    for my $k (@keys) {
+	if ($i != 0) {
+	    $jc->comma ();
+	}
+	$i++;
+	my $error;
+	$error = stringify ($jc, $k);
+	if ($error) {
+	    return $error;
+	}
+	$jc->{output} .= ':';
+	$error = create_json_recursively ($jc, $input->{$k}, \$input->{$k});
+	if ($error) {
+	    return $error;
+	}
+    }
+    $jc->closeB ('}');
+    return undef;
+}
+
+sub create_json_recursively
+{
+    my ($jc, $input, $input_ref) = @_;
+    if ($input_ref) {
+	my $bool = isbool ($input, $input_ref);
+	if ($bool) {
+	    $jc->{output} .= $bool;
+	    return undef;
+	}
+    }
     if (! defined $input) {
 	$jc->{output} .= 'null';
 	return undef;
@@ -328,12 +389,12 @@ sub create_json_recursively
     my $ref = ref ($input);
     if ($ref eq 'JSON::Create::Bool') {
 	if ($$input) {
-	    $jc->{output} .= 'true';
+	    $jc->{output} .=  'true';
 	}
 	else {
-	    $jc->{output} .= 'false';
+	     $jc->{output} .= 'false';
 	}
-	return;
+	return undef;
     }
     if (! keys %{$jc->{_handlers}} && ! $jc->{_obj_handler}) {
 	my $origref = $ref;
@@ -347,62 +408,16 @@ sub create_json_recursively
     }
     if ($ref) {
 	if ($ref eq 'HASH') {
-	    $jc->openB ('{');
-	    my @keys = keys %$input;
-	    if ($jc->{_sort}) {
-		if ($jc->{cmp}) {
-		    @keys = sort {&{$jc->{cmp}} ($a, $b)} @keys;
-		}
-		else {
-		    @keys = sort @keys;
-		}
+	    my $error = $jc->object ($input);
+	    if ($error) {
+		return $error;
 	    }
-	    my $i = 0;
-	    my $n = scalar (@keys);
-	    for my $k (@keys) {
-		my $error = stringify ($jc, $k);
-		if ($error) {
-		    return $error;
-		}
-		$jc->{output} .= ':';
-		my $bool = isbool (\$input->{$k});
-		if ($bool) {
-		    $jc->{output} .= $bool;
-		}
-		else {
-		    my $error = create_json_recursively ($jc, $input->{$k});
-		    if ($error) {
-			return $error;
-		    }
-		}
-		$i++;
-		if ($i < $n) {
-		    $jc->comma ();
-		}
-	    }
-	    $jc->closeB ('}');
 	}
 	elsif ($ref eq 'ARRAY') {
-	    $jc->openB ('[');
-	    my $i = 0;
-	    my $n = scalar (@$input);
-	    for my $k (@$input) {
-		my $bool = isbool (\$k);
-		if ($bool) {
-		    $jc->{output} .= $bool;
-		}
-		else {
-		    my $error = create_json_recursively ($jc, $k);
-		    if ($error) {
-			return $error;
-		    }
-		}
-		$i++;
-		if ($i < $n) {
-		    $jc->comma ();
-		}
+	    my $error = $jc->array ($input);
+	    if ($error) {
+		return $error;
 	    }
-	    $jc->closeB (']');
 	}
 	elsif ($ref eq 'SCALAR') {
 	    if ($jc->{_strict}) {
